import { AutosuggestOptions } from '@bloomreach/discovery-web-sdk';
import { BestsellerOptions } from '@bloomreach/discovery-web-sdk';
import { ButtonHTMLAttributes } from 'react';
import { CategorySearchOptions } from '@bloomreach/discovery-web-sdk';
import { ChangeEventHandler } from 'react';
import { ComponentPropsWithRef } from 'react';
import { Configuration } from '@bloomreach/discovery-web-sdk';
import { ContentSearchOptions } from '@bloomreach/discovery-web-sdk';
import { Context } from 'react';
import { DetailedHTMLProps } from 'react';
import { Dispatch } from 'react';
import { DO_NOT_USE_OR_YOU_WILL_BE_FIRED_CALLBACK_REF_RETURN_VALUES } from 'react';
import { FormEventHandler } from 'react';
import { FormHTMLAttributes } from 'react';
import { ForwardRefExoticComponent } from 'react';
import { HTMLProps } from 'react';
import { InputHTMLAttributes } from 'react';
import { MutableRefObject } from 'react';
import { ProductSearchOptions } from '@bloomreach/discovery-web-sdk';
import { PropsWithChildren } from 'react';
import { ReactElement } from 'react';
import { RefAttributes } from 'react';
import { RefObject } from 'react';
import { SearchResponse } from '@bloomreach/discovery-web-sdk';
import { SetStateAction } from 'react';
import { SliderProps } from '@radix-ui/react-slider';
import { SuggestResponse } from '@bloomreach/discovery-web-sdk';
import { ToggleProps } from '@radix-ui/react-toggle';
import { ToolbarButtonProps } from '@radix-ui/react-toolbar';
import { ToolbarLinkProps } from '@radix-ui/react-toolbar';
import { ToolbarProps } from '@radix-ui/react-toolbar';
import { ToolbarToggleGroupSingleProps } from '@radix-ui/react-toolbar';
import { ToolbarToggleItemProps } from '@radix-ui/react-toolbar';
import { UseFloatingReturn } from '@floating-ui/react';
import { UseInteractionsReturn } from '@floating-ui/react';

export declare const AutoSuggestContext: Context<AutoSuggestContextType | null>;

export declare const AutoSuggestContextProvider: ({ children }: PropsWithChildren) => ReactElement;

export declare type AutoSuggestContextType = {
    error: unknown;
    setError: Dispatch<SetStateAction<unknown>>;
    loading: boolean;
    setLoading: Dispatch<SetStateAction<boolean>>;
    suggestResponse: SuggestResponse | null;
    setSuggestResponse: Dispatch<SetStateAction<SuggestResponse | null>>;
};

/**
 * Creates a debounced function that delays invoking the provided function until after the specified
 * wait time has elapsed since the last time the debounced function was invoked.
 *
 * @param func - The function to debounce.
 * @param wait - The number of milliseconds to delay.
 * @returns A new debounced function.
 */
export declare function debounce<T extends (...args: any[]) => any>(func: T, wait: number): (...args: Parameters<T>) => void;

export declare const FloatingUIContext: Context<FloatingUIContextType | null>;

export declare const FloatingUIContextProvider: ({ children }: PropsWithChildren) => ReactElement;

export declare type FloatingUIContextType = Pick<UseFloatingReturn<Element>, 'refs' | 'floatingStyles' | 'context'> & UseInteractionsReturn & {
    open: boolean;
    setOpen: Dispatch<SetStateAction<boolean>>;
    activeIndex: number | null;
    setActiveIndex: Dispatch<SetStateAction<number | null>>;
    handleInputChange: ChangeEventHandler<HTMLInputElement>;
    handleQuerySelect: () => void;
    handleProductSelect: () => void;
    listRef: MutableRefObject<Array<HTMLElement | null>>;
};

export declare const formatPrice: (price: number, currency?: string, locale?: string) => string | number;

export declare type ItemsPerPageOptionsType = Array<number | {
    label: string;
    value: number;
}>;

export declare const LimitlessUIProvider: ({ children }: PropsWithChildren) => ReactElement;

export declare type PageNavigationItemsListType = Array<PageNavigationItemType>;

export declare type PageNavigationItemType = number | '...';

/**
 * Pagination component to display pagination links and manage the number of items per page
 *
 * ### Usage
 * ```tsx
 * import { Pagination } from '@bloomreach/react-banana-ui'
 *
 * <Pagination
 *   count={100}
 *   itemsPerPage={10}
 *   page={0}
 *   onPageChange={(page) => console.log(page)}
 *   itemsPerPageOptions={[10, 25, 50]}
 *   onItemsPerPageChange={(num) => console.log(num)}
 * />
 * ```
 */
export declare const Pagination: ForwardRefExoticComponent<PaginationProps & RefAttributes<HTMLDivElement>>;

export declare interface PaginationProps {
    /**
     * Custom class name for the container of this component
     */
    className?: string;
    /**
     * Total number of items
     */
    count: number;
    /**
     * Zero-based index of the current page
     */
    page: number;
    /**
     * Number of items per page.
     */
    itemsPerPage: number;
    /**
     * Callback fired when the page is changed.
     */
    onPageChange?: (page: number) => void;
    /**
     * Customizes the options of the items per page select field.
     * If less than two options are available, no select field will be displayed
     */
    itemsPerPageOptions?: ItemsPerPageOptionsType;
    /**
     * Callback fired when the number of items per page is changed
     */
    onItemsPerPageChange?: (itemsPerPage: number) => void;
}

export declare const ProductCard: {
    Root: ForwardRefExoticComponent<Omit<ProductCardProps, "ref"> & RefAttributes<HTMLDivElement>>;
    Header: ForwardRefExoticComponent<Omit<ProductCardProps, "ref"> & RefAttributes<HTMLDivElement>>;
    Image: ForwardRefExoticComponent<Omit<ProductCardImageProps, "ref"> & RefAttributes<HTMLImageElement>>;
    Body: ForwardRefExoticComponent<Omit<ProductCardProps, "ref"> & RefAttributes<HTMLDivElement>>;
    Footer: ForwardRefExoticComponent<Omit<ProductCardProps, "ref"> & RefAttributes<HTMLDivElement>>;
    Title: ForwardRefExoticComponent<Omit<ProductCardProps, "ref"> & RefAttributes<HTMLDivElement>>;
    SubTitle: ForwardRefExoticComponent<Omit<ProductCardProps, "ref"> & RefAttributes<HTMLDivElement>>;
    Label: ForwardRefExoticComponent<Omit<ProductCardProps, "ref"> & RefAttributes<HTMLDivElement>>;
    Price: ForwardRefExoticComponent<Omit<ProductCardPriceProps, "ref"> & RefAttributes<HTMLDivElement>>;
    PriceRange: ForwardRefExoticComponent<Omit<ProductCardPriceRangeProps, "ref"> & RefAttributes<HTMLDivElement>>;
    Button: ForwardRefExoticComponent<ProductCardButtonProps & RefAttributes<HTMLButtonElement>>;
    Badge: ForwardRefExoticComponent<Omit<ProductCardProps, "ref"> & RefAttributes<HTMLDivElement>>;
    FavoriteButton: ForwardRefExoticComponent<ToggleProps & RefAttributes<HTMLButtonElement>>;
};

declare interface ProductCardButtonProps extends PropsWithChildren, ButtonHTMLAttributes<HTMLButtonElement> {
    /**
     * Custom class name for the container of the component.
     */
    className?: string;
    variant?: string;
}

export declare interface ProductCardImageProps extends HTMLProps<HTMLImageElement> {
    className?: string;
    src: string;
    alt: string;
}

export declare interface ProductCardPriceProps extends ProductCardProps {
    price: number;
    salePrice?: number;
    currency?: string;
    locale?: string;
}

export declare interface ProductCardPriceRangeProps extends ProductCardProps {
    from: number;
    to: number;
    currency?: string;
    locale?: string;
}

export declare interface ProductCardProps extends PropsWithChildren, HTMLProps<HTMLDivElement> {
    /**
     * Custom class name for the container of the component.
     */
    className?: string;
}

declare const Range_2: {
    Root: ForwardRefExoticComponent<Omit<RangeRootProps, "ref"> & RefAttributes<HTMLDivElement>>;
    Provider: ({ children, value }: RangeProviderProps) => ReactElement;
    MinInput: ForwardRefExoticComponent<Omit<RangeInputProps, "ref"> & RefAttributes<HTMLInputElement>>;
    MaxInput: ForwardRefExoticComponent<Omit<RangeInputProps, "ref"> & RefAttributes<HTMLInputElement>>;
    Slider: ForwardRefExoticComponent<SliderProps & RefAttributes<HTMLSpanElement>>;
    Inputs: ForwardRefExoticComponent<Omit<RangeInputsProps, "ref"> & RefAttributes<HTMLDivElement>>;
    Separator: ForwardRefExoticComponent<Omit<RangeInputsProps, "ref"> & RefAttributes<HTMLDivElement>>;
    UpdateButton: ForwardRefExoticComponent<RangeButtonProps & RefAttributes<HTMLButtonElement>>;
};
export { Range_2 as Range }

export declare interface RangeButtonProps extends PropsWithChildren, ButtonHTMLAttributes<HTMLButtonElement> {
    /**
     * Custom class name for the container of the component.
     */
    className?: string;
}

export declare interface RangeContextProps extends RangeProps {
    onInputChange: (index: number, value: string) => void;
    onSliderChange: (value: [number, number]) => void;
    onUpdate: () => void;
}

export declare interface RangeInputProps extends Omit<HTMLProps<HTMLInputElement>, 'type' | 'defaultValue' | 'min' | 'max' | 'step' | 'onChange'> {
    /**
     * Custom class name for the container of the component.
     */
    className?: string;
}

export declare interface RangeInputsProps extends PropsWithChildren, HTMLProps<HTMLDivElement> {
    /**
     * Custom class name for the container of the component.
     */
    className?: string;
}

export declare interface RangeProps {
    /**
     * Minimum value
     */
    min: number;
    /**
     * Maximum value
     */
    max: number;
    /**
     * Step increments to change value
     */
    step?: number;
    /**
     * Selected range value
     */
    value: [number, number];
    /**
     * onChange event
     */
    onChange?: (value: [number, number]) => void;
    /**
     * If true, trigger update on every change. Else, trigger change manually using the update button/api
     */
    autoUpdate?: boolean;
}

export declare interface RangeProviderProps extends PropsWithChildren {
    value: RangeProps;
}

export declare interface RangeRootProps extends PropsWithChildren, RangeProps, Omit<HTMLProps<HTMLDivElement>, 'min' | 'max' | 'value' | 'step' | 'onChange'> {
    /**
     * Custom class name for the container of the component.
     */
    className?: string;
}

/**
 * A search box component to interface with the Bloomreach Discovery search
 * functionality
 */
export declare const SearchBox: ForwardRefExoticComponent<Pick<Omit<DetailedHTMLProps<FormHTMLAttributes<HTMLFormElement>, HTMLFormElement>, "ref"> & {
ref?: ((instance: HTMLFormElement | null) => void | DO_NOT_USE_OR_YOU_WILL_BE_FIRED_CALLBACK_REF_RETURN_VALUES[keyof DO_NOT_USE_OR_YOU_WILL_BE_FIRED_CALLBACK_REF_RETURN_VALUES]) | RefObject<HTMLFormElement> | null | undefined;
}, "onReset" | "onSubmit"> & Pick<Omit<DetailedHTMLProps<InputHTMLAttributes<HTMLInputElement>, HTMLInputElement>, "ref"> & {
ref?: ((instance: HTMLInputElement | null) => void | DO_NOT_USE_OR_YOU_WILL_BE_FIRED_CALLBACK_REF_RETURN_VALUES[keyof DO_NOT_USE_OR_YOU_WILL_BE_FIRED_CALLBACK_REF_RETURN_VALUES]) | RefObject<HTMLInputElement> | null | undefined;
}, "name" | "onChange"> & {
configuration: Configuration;
searchOptions: Omit<ProductSearchOptions, "q"> | Omit<ContentSearchOptions, "q"> | Omit<BestsellerOptions, "q"> | Omit<CategorySearchOptions, "q">;
suggestOptions?: Omit<AutosuggestOptions, "q">;
searchType: SearchType;
autoQuery?: boolean;
debounceDelay?: number;
classNames?: Partial<Record<SearchBoxClassElement, string>>;
labels?: Partial<Record<SearchBoxLabelElement, string>>;
submitIcon?: () => ReactElement;
resetIcon?: () => ReactElement;
} & RefAttributes<HTMLFormElement>>;

/**
 * Elements that can recieve additional classNames
 */
export declare type SearchBoxClassElement = 'form' | 'input' | 'label' | 'submit' | 'submitIcon' | 'reset' | 'resetIcon';

/**
 * Elements that can recieve additional classNames
 */
export declare type SearchBoxLabelElement = 'placeholder' | 'label' | 'submit' | 'reset';

export declare type SearchBoxProps = Pick<ComponentPropsWithRef<'form'>, 'onSubmit' | 'onReset'> & Pick<ComponentPropsWithRef<'input'>, 'onChange' | 'name'> & {
    /**
     * The Configuration for creating a Bloomreach search integration
     */
    configuration: Configuration;
    /**
     * The options specific to a Bloormeach search API e.g. `q` and `fl`
     */
    searchOptions: Omit<ProductSearchOptions, 'q'> | Omit<ContentSearchOptions, 'q'> | Omit<BestsellerOptions, 'q'> | Omit<CategorySearchOptions, 'q'>;
    /**
     * The options specific to the Autosuggest API
     */
    suggestOptions?: Omit<AutosuggestOptions, 'q'>;
    /**
     * The type of search.
     */
    searchType: SearchType;
    /**
     * Enable autoQuery
     */
    autoQuery?: boolean;
    /**
     * The number of miliseconds the auto-search should be debounced
     */
    debounceDelay?: number;
    /**
     * Classnames to be added to their respective elements, e.g. input or submit button
     */
    classNames?: Partial<Record<SearchBoxClassElement, string>>;
    /**
     * Text to be added to their respective elements, e.g. input label or submit button
     */
    labels?: Partial<Record<SearchBoxLabelElement, string>>;
    /**
     * Icon to be added to the Submit button
     */
    submitIcon?: () => ReactElement;
    /**
     * Icon to be added to the Reset button
     */
    resetIcon?: () => ReactElement;
};

export declare const SearchContext: Context<SearchContextType | null>;

export declare const SearchContextProvider: ({ children }: PropsWithChildren) => ReactElement;

export declare type SearchContextType = {
    error: unknown;
    loading: boolean;
    searchResponse: SearchResponse | null;
    inputValue: string;
    setInputValue: Dispatch<SetStateAction<string>>;
    setError: Dispatch<SetStateAction<unknown>>;
    setLoading: Dispatch<SetStateAction<boolean>>;
    setSearchResponse: Dispatch<SetStateAction<SearchResponse | null>>;
};

/**
 * The type of search.
 */
export declare type SearchType = 'product' | 'category' | 'content' | 'bestseller';

/**
 * Renders suggestions returned from the Autosuggest API based on a query
 */
export declare const Suggestions: (props: SuggestionsProps) => ReactElement;

export declare type SuggestionsProps = {
    /**
     * The number of miliseconds the auto-search should be debounced
     */
    debounceDelay?: number;
    /**
     * The Configuration for creating a Bloomreach integration
     */
    configuration: Configuration;
    /**
     * The options specific to a Bloormeach suggestions request
     */
    suggestOptions: Omit<AutosuggestOptions, 'q'>;
    /**
     * The query to provide suggestions for
     */
    inputValue: string;
};

export declare const SwatchBar: {
    Root: ForwardRefExoticComponent<SwatchBarProps & RefAttributes<HTMLDivElement>>;
    SwatchGroup: ForwardRefExoticComponent<Omit<SwatchBarSwatchGroupProps & RefAttributes<HTMLDivElement>, "ref"> & RefAttributes<HTMLDivElement>>;
    Link: ForwardRefExoticComponent<SwatchBarLinkProps & RefAttributes<HTMLAnchorElement>>;
    Button: ForwardRefExoticComponent<SwatchBarButtonProps & RefAttributes<HTMLButtonElement>>;
    SwatchItem: ForwardRefExoticComponent<SwatchItemProps & RefAttributes<HTMLButtonElement>>;
    SwatchColor: ForwardRefExoticComponent<SwatchColorProps & RefAttributes<HTMLButtonElement>>;
    SwatchImage: ForwardRefExoticComponent<SwatchItemProps & RefAttributes<HTMLButtonElement>>;
    SwatchText: ForwardRefExoticComponent<SwatchItemProps & RefAttributes<HTMLButtonElement>>;
};

export declare interface SwatchBarButtonProps extends ToolbarButtonProps {
}

export declare interface SwatchBarLinkProps extends ToolbarLinkProps {
}

export declare interface SwatchBarProps extends ToolbarProps {
}

export declare interface SwatchBarSwatchGroupProps extends Omit<ToolbarToggleGroupSingleProps, 'type'> {
}

export declare interface SwatchColorProps extends SwatchItemProps {
    /**
     * Color for the swatch to be rendered
     */
    color: string;
}

export declare interface SwatchItemProps extends ToolbarToggleItemProps {
}

/**
 * Provides a default theme for the Limitless UI Components
 *
 * ### Usage
 *
 * ```tsx
 * import { Theme } from '@bloomreach/limitless-ui-react';
 *
 * <Theme dir={dir} locale={locale} currency={currency}>
 *   {children}
 * </Theme>
 * ```
 */
export declare const Theme: ForwardRefExoticComponent<Omit<ThemeProps, "ref"> & RefAttributes<HTMLDivElement>>;

export declare interface ThemeContextProps {
    /**
     * Direction of text on the page
     */
    dir?: 'ltr' | 'rtl';
    /**
     * A valid locale identifier. Eg., en-US, ja-JP, ar-AE, hi-IN, zh-CN
     */
    locale?: string;
    /**
     * A valid ISO 4217 currency code. Eg., USD, EUR, INR, AED, CNY, JPY
     */
    currency?: string;
}

export declare interface ThemeProps extends PropsWithChildren, ThemeContextProps, Omit<HTMLProps<HTMLDivElement>, 'dir'> {
    className?: string;
}

export declare const ThemeProvider: ({ children, value }: ThemeProviderProps) => ReactElement;

export declare interface ThemeProviderProps extends PropsWithChildren {
    value: ThemeContextProps;
}

export declare function useAutoSuggest(configuration: Configuration, suggestOptions: AutosuggestOptions): useAutoSuggest_2;

declare function useAutoSuggest(configuration: Configuration, suggestOptions: AutosuggestOptions): useAutoSuggest_2;

declare type useAutoSuggest_2 = {
    response: SuggestResponse | null;
    loading: boolean;
    error: unknown;
};

export declare function useRange(): RangeContextProps | null;

/**
 * Custom hook to perform a product search using Bloomreach Discovery Web SDK.
 *
 * @param searchType - The type of search to be executed.
 * @param configuration - The configuration object for the Bloomreach SDK.
 * @param searchOptions - Additional search options excluding the query.
 * @returns An object containing the search response, loading state, and any error encountered.
 */
export declare function useSearch(searchType: SearchType, configuration: Configuration, searchOptions: UseSearchOptions): UseSearchResponse;

declare type UseSearchBox = {
    response: SearchResponse | null;
    loading: boolean;
    error: unknown;
    changeHandler: ChangeEventHandler<HTMLInputElement>;
    submitHandler: FormEventHandler<HTMLFormElement>;
    resetHandler: FormEventHandler<HTMLFormElement>;
    inputValue: string;
};

export declare function useSearchBox(props: SearchBoxProps): UseSearchBox;

export declare type UseSearchOptions = ProductSearchOptions | CategorySearchOptions | ContentSearchOptions | BestsellerOptions;

export declare type UseSearchResponse = {
    response: SearchResponse | null;
    loading: boolean;
    error: unknown;
};

declare type UseSuggestions = {
    response: SuggestResponse | null;
    loading: boolean;
    error: unknown;
};

export declare function useSuggestions(props: SuggestionsProps, inputValue: string): UseSuggestions;

export declare function useTheme(): ThemeContextProps;

export { }
